<?php

namespace Center\Capsule\Controllers\Requests;

use Center\Capsule\Services\Property\Preset\Conf\DateProperty;
use Center\Capsule\Services\Property\Preset\Conf\LocateProperty;
use Center\Capsule\Services\Property\Preset\Conf\NumberProperty;
use Center\Capsule\Services\Property\Preset\Conf\VarCharProperty;
use Illuminate\Validation\Rule;

class PropertyRequest extends BaseRequest
{
    public function authorize()
    {
        return true;
    }

    private function getType()
    {
        return strtolower($this->input('type'));
    }


    public static function onlyField()
    {
        return [
            "application_key",
            "name",
            "visibility",
            "widget"
        ];
    }

    public function rules()
    {
        $name = strtolower($this->route()->getName());
        if (isset($this->ruleMap()[$name])) {
            return $this->ruleMap()[$name];
        }
        return [];
    }


    /**
     * messages
     *
     * @return array
     */
    public function messages()
    {
        switch ($this->route()->getName()) {
            case 'property@create':
                return [
                    'name.required'        => 'property_name_required',
                    'type.required'        => 'type_required',
                    'option.required'      => 'option_required',
                    'option.type.required' => 'option_type_required',
                    //property.visibility
                    'type.relation_type'   => 'relation_type_error',
                    'widget.array'         => 'widget_json'
                ];
                break;
            case 'property@update':
                return [
                    'required'           => 'property_required',
                    'name.required'      => 'property_name_required',
                    'visibility.numeric' => 'visibility_number',
                    'widget.array'       => 'widget_json'
                ];
                break;
            case 'property@delete':
                return [];
                break;
            default:
                return [
                    'default' => 'property_default'
                ];
                break;
        }
    }


    public function ruleMap()
    {
        $type = $this->getType();
        return [
            'property@create' => $this->createRule($type),
            'property@update' => $this->updateRule()
        ];
    }

    /**
     * createRule
     *
     * @param $type
     * @return array|mixed
     */
    private function createRule($type)
    {

        $map = [
            'varchar'    => $this->varCharRule(),
            'number'     => $this->numberRule(),
            'date'       => $this->dateRule(),
            'relation'   => $this->relationRule(),
            'json'       => $this->jsonRule(),
            'text'       => $this->textRule(),
            'attachment' => $this->attachmentRule(),
            'locateRule' => $this->locateRule()
        ];
        return isset($map[$type]) ? $map[$type] : $this->defaultRule();
    }

    public function defaultRule()
    {
        return [
            'name' => 'required',
            'type' => 'required'
        ];
    }

    private function jsonRule()
    {
        return $this->defaultRule();
    }

    private function textRule()
    {
        return [
            'name'   => 'required',
            'type'   => 'required',
            //'option' => 'required',
            'widget' => 'array'
        ];
    }

    private function dateRule()
    {
        return [
            'name'        => 'required',
            'type'        => 'required',
            'option'      => 'required',
            'widget'      => 'array',
            'option.type' => [
                function ($attribute, $value, $fail) {
                    $type       = array_keys($value);
                    $allow_type = DateProperty::getOptionType();
                    if (count($value) > 1 || !isset($allow_type[reset($type)])) {
                        $fail('option_type_required');
                    }

                }
            ]
        ];
    }

    private function numberRule()
    {
        return [
            'name'        => 'required',
            'type'        => 'required',
            'widget'      => 'array',
            'option'      => 'required',
            'option.type' => [
                'required',
                function ($attribute, $value, $fail) {
                    $type       = array_keys($value);
                    $allow_type = NumberProperty::getOptionType();
                    if (count($value) > 1 || !isset($allow_type[reset($type)])) {
                        $fail('option_type_required');
                    }

                }

            ]
        ];
    }

    private function varCharRule()
    {
        return [
            'name'        => 'required',
            'type'        => 'required',
            'option'      => 'required',
            'widget'      => 'array',
            'option.type' => [
                function ($attribute, $value, $fail) {
                    $type       = array_keys($value);
                    $allow_type = VarCharProperty::getOptionType();
                    if (count($value) > 1 || !isset($allow_type[reset($type)])) {
                        $fail('option_type_required');
                    }

                }
            ]
        ];
    }

    private function relationRule()
    {
        return [
            'name'                      => 'required',
            'type'                      => 'required',
            'option.relation_model_key' => 'required',
            'widget'                    => 'array',
            'option.relation_type'      => [
                'required',
                Rule::in(['one', 'many']),
            ]
        ];
    }

    private function attachmentRule()
    {
        return [
            'name'         => 'required',
            'type'         => 'required',
            'widget'       => 'array',
            'option'       => 'required',
            'option.ext'   => 'required|array',
            'option.multi' => [
                'required',
                Rule::in([1, 2]),
            ],
            'option.size'  => 'required|integer'
        ];
    }

    private function locateRule()
    {
        return [
            'name'        => 'required',
            'type'        => 'required',
            'widget'      => 'array',
            'option'      => 'required',
            'option.type' => [
                function ($attribute, $value, $fail) {
                    $type       = array_keys($value);
                    $allow_type = LocateProperty::getOptionType();
                    if (count($value) > 1 || !isset($allow_type[reset($type)])) {
                        $fail('option_type_required');
                    }

                }
            ]
        ];
    }


    private function updateRule()
    {
        return [
            'name'       => 'required',
            'visibility' => [
                'numeric',
                Rule::in([1, 2]),
            ],
            'widget'     => 'array'
        ];
    }


}