<?php

namespace Center\Capsule\Services\Models;

use Center\Capsule\Constants\ModelsConst;
use Center\Capsule\Models\BaseModel;
use Center\Capsule\Models\Models;
use Center\Capsule\Services\Extra\RequestService;
use Center\Capsule\Services\Models\Assist\Application;
use Exception;
use Illuminate\Support\Facades\DB;
use Center\Capsule\Models\ModelProperties;
use Center\Capsule\Models\ApplicationModels;

class ModelsService
{
    public $modelName = '';

    public $modelKey = '';

    /**
     * 通过模型key和应用key获取模型下的属性
     *
     * @param $data
     * @return array
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function getProperties($data)
    {
        $application_key = $data['application_key'];
        $model_key       = $data['model_key'];
        $count           = isset($data['count']) ? intval($data['count']) : ModelsConst::$pageCount;
        $application_id  = Application::getApplicationIdByKey($application_key);
        $systematic      = isset($data['systematic']);
        $res             = [];
        if ($application_id) {
            $model_id = $this->getModelId($application_id, $model_key);
            if ($model_id) {
                $res = ModelProperties::query()
                    ->select(array_merge(ModelProperties::selectMap(), ['id']))
                    ->where('model_id', $model_id);
                if ($systematic) {
                    $res->where('property_type', '<>', 'systematic');
                }
                $res = $res->orderBy('sort', 'desc')
                      ->paginate($count);
            }
        }
        return $res;
    }

    /**
     * 通过应用id和模型key获取模型id
     *
     * @param $application_id
     * @param $model_key
     * @return int
     */
    public function getModelId($application_id, $model_key)
    {
        return Models::query()->where([
                'application_id' => $application_id,
                'model_key'      => $model_key
            ]
        )->value('id');
    }

    /**
     * create
     *
     * @param $application_key
     * @param $name
     * @return array|bool
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function create($application_key, $name)
    {
        $this->modelName = $name;
        $application     = Application::getApplicationIdByKey($application_key, 'all');
        if (!$application) {
            return false;
        }
        $application_id = $application['id'];
        $model_key      = $this->uuid();
        $res            = Models::query()->where([
            'model_key'      => $model_key,
            'application_id' => $application_id
        ])->first();
        if ($res) {
            return false;
        }
        $data             = [
            'application_id' => $application_id,
            'model_key'      => $model_key,
            'model_name'     => $name,
            'model_status'   => ModelsConst::$modelStatus['enable']['value'],
        ];
        $application_name = $application['name'];
        $perm_data        = [
            '{application_key}'  => $application_key,
            '{application_name}' => $application_name,
            '{model_key}'        => $model_key,
            '{model_name}'       => $name
        ];
        DB::transaction(function () use ($data, $perm_data, $application_id, $model_key) {
            $model = Models::query()->create($data);
            if (!$model) {
                throw new Exception("insert_model_fail");
            }
            $data = [
                'application_id' => $application_id,
                'model_id'       => $model->id,
            ];
            $res  = ApplicationModels::query()->create($data);
            if (!$res) {
                throw new Exception("insert_model_fail");
            }

            //推送权限中心
//            $res = $this->pushPermission($perm_data);
//            if (!$res) {
//                throw new \Exception("insert_model_fail");
//            }

            $this->createDynamicModel(ModelsConst::COMMON_MODEL_FILE_NAME)
                ->writeSystematic($model->getAttribute('id'));
        });
        return [
            'model_key'  => $model_key,
            'model_name' => $name
        ];
    }

    /**
     * createDynamicModel
     *
     * @param $common_file
     * @return mixed
     */
    private function createDynamicModel($common_file)
    {
        $model_key    = $this->modelKey;
        $common_model = $this->getCommonModel($common_file);
        if (is_file($common_model)) {
            $content   = file_get_contents($common_model);
            $model_dir = $this->getModelDir();
            file_put_contents($model_dir.'/'.$model_key.'.php',
                str_replace(['{{MODEL_KEY}}', '{{MODEL_NAME}}'], [
                    $model_key,
                    $this->modelName
                ], $content));
            /**
             * @var BaseModel $class
             */
            $class = ModelsConst::MODEL_NAMESPACE_PREFIX."\\".$model_key;
            return (new $class())->createTable();
        }
    }

    /**
     * getCommonModel
     *
     * @param $common_file
     * @return string
     */
    private function getCommonModel($common_file)
    {
        return __DIR__.ModelsConst::COMMON_MODEL_FILE_PATH.$common_file;
    }

    /**
     * getModelDir
     *
     * @return string
     */
    private function getModelDir()
    {
        $dir = app_path().ModelsConst::MODEL_DIR;
        if (!is_dir($dir)) {
            mkdir($dir, 0777, true);
        }
        return $dir;
    }

    /**
     * 随机数生成
     */
    protected function uuid()
    {
        $this->modelKey = uniqid('Model');
        return $this->modelKey;
    }

    /**
     * update
     *
     * @param $data
     * @return array|bool
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function update($data)
    {
        $application_key = $data['application_key'];
        $model_key       = $data['model_key'];
        $name            = $data['name'];

        $application = Application::getApplicationIdByKey($application_key, 'all');
        if (!$application || !$this->getModelId($application['id'], $model_key)) {
            return false;
        }
        $application_id   = $application['id'];
        $application_name = $application['name'];
        $perm_data        = [
            '{application_key}'  => $application_key,
            '{application_name}' => $application_name,
            '{model_key}'        => $model_key,
            '{model_name}'       => $name
        ];

        DB::beginTransaction();
        try {
            $res = Models::query()->where([
                'model_key'      => $model_key,
                'application_id' => $application_id
            ])->update(['model_name' => $name]);
            if (!$res) {
                throw new Exception("update_model_fail");
            }
            $res = $this->pushPermission($perm_data);
            if (!$res) {
                throw new Exception("update_model_fail");
            }
        } catch (Exception $e) {
            DB::rollBack();
            return false;
        }
        DB::commit();

        return ['model_key' => $model_key, 'model_name' => $name];
    }


    /**
     * show
     *
     * @param $data
     * @return mixed
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function show($data)
    {
        $count = isset($data['count']) ? intval($data['count']) : ModelsConst::$pageCount;
        $field = [
            'model_key',
            'model_name',
            'model_status',
            'created_uid',
            'application_id'
        ];
        $model = Models::query()->select($field);
        if (isset($data['application_key'])) {
            $application_key = $data['application_key'];
            $app             = Application::getApplicationIdByKey($application_key, 'all');
            $application     = [
                $app['id'] => [
                    'application_name' => $app['name'],
                    'application_key'  => $app['application_key']
                ]
            ];
            $model           = $model->where('application_id', $app['id']);
        } else {
            $application = $this->getApplicationIds();
            $model       = $model->whereIn('application_id', array_keys($application));
        }
        if (isset($data['model_name'])) {
            $model = $model->modelName($data['model_name']);
        }
        if (isset($data['model_status'])) {
            $model = $model->modelStatus($data['model_status']);
        }

        $model = $model->orderBy('models.created_at', 'desc')->paginate($count)->toArray();
        if ($model['data']) {
            $info = $model['data'];
            foreach ($info as &$v) {
                $v['application_key']  = $application[$v['application_id']]['application_key'];
                $v['application_name'] = $application[$v['application_id']]['application_name'];
            }
            $model['data'] = $info;
        }

        return $model;
    }

    /**
     * getApplicationIds
     *
     * @return array
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    protected function getApplicationIds()
    {
        $config   = config('capsule');
        $uri      = Application::getApplicationHost().$config['application']['get_application_list'];
        $response = RequestService::getInstance()->get($uri);
        $ids      = [];
        if ($response->isSuccess()) {
            $data = $response->getResult();
            if ($data) {
                foreach ($data as $v) {
                    $ids[intval($v['id'])] = [
                        'application_key'  => $v['application_key'],
                        'application_name' => $v['name']
                    ];
                }
            }
        }
        return $ids;
    }

    /**
     * delete
     *
     * @param $application_key
     * @param $model_key
     * @return array|bool
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function delete($application_key, $model_key)
    {
        $application_id = Application::getApplicationIdByKey($application_key);
        $model_id       = $this->getModelId($application_id, $model_key);
        if (!$application_id || !$model_id) {
            return false;
        }

        DB::transaction(function () use ($application_id, $model_key, $model_id) {
            $res = Models::query()->where(['application_id' => $application_id, 'model_key' => $model_key])->delete();
            if (!$res) {
                throw new Exception("delete_model_fail");
            }
            $res = ApplicationModels::query()->where([
                    'application_id' => $application_id,
                    'model_id'       => $model_id
                ]
            )->delete();
            if (!$res) {
                throw new Exception("delete_model_fail");
            }
            $properties = ModelProperties::query()->where('model_id', $model_id)->get();
            if (!$properties->isEmpty()) {
                $res = ModelProperties::query()->where('model_id', $model_id)->delete();
                if (!$res) {
                    throw new Exception("delete_model_fail");
                }
            }
        });
        return ['model_key' => $model_key];
    }

    /**
     * detail
     *
     * @param $application_key
     * @param $model_key
     * @return array
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function detail($application_key, $model_key)
    {
        $application_id = Application::getApplicationIdByKey($application_key);
        if (!$application_id) {
            return [];
        }
        $info = Models::query()->select('model_key', 'model_name')->where([
            'application_id' => $application_id,
            'model_key'      => $model_key
        ])->first();
        if ($info) {
            $info->application_key = $application_key;
        }
        return $info->toArray();
    }

    /**
     * propertyDetail
     *
     * @param $model_key
     * @param $property_key
     * @return mixed
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function propertyDetail($model_key, $property_key)
    {
        $model       = Models::query()->where('model_key', $model_key)->firstOrFail();
        $application = Application::getApplicationIdByKey($model->application_id, 'all', 'id');
        $property    = ModelProperties::query()->select(array_merge(ModelProperties::selectMap(), ['id']))
            ->where([
                'model_id'     => $model->id,
                'property_key' => $property_key
            ])->firstOrFail();
        if ($application) {
            $property->application_key = $application['application_key'];
        }
        return $property;
    }


    /**
     * pushPermission
     *
     * @param $data
     * @return bool
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    private function pushPermission($data)
    {
        $content  = $this->getBasePermissionFile();
        $content  = str_replace(array_keys($data), array_values($data), $content);
        $config   = config('capsule');
        $host     = Application::getApplicationHost('user');
        $url      = $config['permission']['initial_action'];
        $uri      = $host.$url;
        $response = RequestService::getInstance()->post($uri, [
            'action' => $content
        ]);
        if ($response->isSuccess()) {
            return true;
        }
        return false;
    }

    /**
     * getBasePermissionFile
     *
     * @return false|string
     */
    private function getBasePermissionFile()
    {
        $file = __DIR__.ModelsConst::COMMON_MODEL_FILE_PATH.ModelsConst::PERM_DATA_FILE_NAME;
        if (is_file($file)) {
            return file_get_contents($file);
        }
        return '';
    }
}