<?php

namespace Center\Capsule\Services\Property\Build;

use App\Consts\PropertyConst;
use Center\Capsule\Models\ModelProperties;
use Center\Capsule\Models\PresetProperties;
use Center\Capsule\Services\Property\Build\Traits\ConvertTrait;
use Center\Capsule\Services\Property\Exception\PropertyException;
use Center\Capsule\Services\Property\Interfaces\PropertyInterface;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;

/**
 * @method string getName
 * @method string getOption
 * @method string getFilter
 * @method string getModelKey
 */

abstract class BaseProperty implements PropertyInterface
{
    use ConvertTrait;
    /**
     * @var Blueprint
     */
    public $schemaTable = null;

    public $key;

    public $modelKey;

    public $name;

    public $type;

    public $filter;

    public $option;

    public $visibility = 1;

    public $widget = null;

    protected $originData;

    protected $waitBind = [
        'key',
        'name',
        'filter',
        'option',
        'visibility',
        'modelKey',
        'widget'
    ];

    public $changeList = [];

    protected $dependentData = [];

    protected $allowCallMethod = [
        'getName',
        'getOption',
        'getFilter',
        'getModelKey',
        'getWidget'
    ];


    public function init(array $data)
    {
        $this->dependentData = $data;
        $this->modelKey      = $data['model_key'];
        $this->bindPreset();
        if ($this->type) {
            $this->originData = PresetProperties::where('property_type', $this->type)->first();
        }
        return $this;
    }

    /**
     * modifyInit
     *
     * @param array $data
     * @return BaseProperty
     * @throws \Center\Capsule\Services\Property\Exception\PropertyException
     */
    public function modifyInit(array $data)
    {
        $origin = ModelProperties::with('models')
            ->where('property_key', $data['key'])
            ->whereHas('models', function (Builder $query) use ($data) {
                $query->where('model_key', $data['model_key']);
            })->select(ModelProperties::selectMap())
            ->first()->toArray();
        if(!empty($origin)){
            $this->convert($data, $origin);
            return $this->init($data);
        }
        throw new PropertyException('origin data exception');
    }

    /**
     * bindPreset
     */
    protected function bindPreset()
    {
        foreach ($this->waitBind as $b) {
            if (property_exists($this, $b) && isset($this->dependentData[$b])) {
                $this->{$b} = $this->dependentData[$b];
            }
        }
        $this->key = $this->getKey();
        $this->diff();
    }

    /**
     * diff
     */
    protected function diff()
    {
        foreach ($this->waitBind as $b) {
            if (!isset($this->dependentData[$b])) {
                array_push($this->changeList, $b);
            }
        }
    }

    /**
     * 获取key
     *
     * @return mixed|string
     */
    public function getKey()
    {
        if (empty($this->dependentData['key'])) {
            $this->key = empty($this->key) ? uniqid("field") : $this->key;
        } else {
            $this->key = $this->dependentData['key'];
        }
        return $this->key;
    }

    public function getVisibility()
    {
        $this->visibility = PropertyConst::$visibility['show']['value'];
        return $this->visibility;
    }


    public function getPropertyModelByKey($model)
    {
        if (!empty($this->key)) {
            return $model::with('models')->where('property_key', $this->key)
                ->whereHas('models', function (Builder $query) {
                    $query->where('model_key', $this->modelKey);
                })->first();
        }
        return $model;
    }

    /**
     *  delete
     *
     * @return bool
     * @throws PropertyException
     */
    public function delete()
    {
        $result = ModelProperties::with('models')
            ->where('property_key', $this->key)
            ->where('systematic', PropertyConst::$systematic['disable']['value'])
            ->whereHas('models', function (Builder $query) {
                $query->where('model_key', $this->modelKey);
            })->delete();
        if ($result) {
            return true;
        }
        throw new PropertyException('delete Property fail');
    }

    /**
     * 魔术方法
     *
     * @param $name
     * @param $arguments
     * @return mixed
     * @throws PropertyException
     */
    public function __call($name, $arguments)
    {
        if (in_array($name, $this->allowCallMethod)) {
            return $this->$name(...$arguments);
        }
        throw new PropertyException('method is not exist');
    }
}