<?php

namespace Center\Capsule\Services\Property\Build;

use Center\Capsule\Models\ModelProperties;
use Center\Capsule\Services\Property\Build\Traits\AlterTrait;
use Center\Capsule\Services\Property\Exception\PropertyException;
use Center\Capsule\Services\Property\Preset\Conf\DateProperty as Conf;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Database\Schema\ColumnDefinition;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;
use Illuminate\Support\Facades\Validator;

class DateProperty extends BaseProperty
{
    use AlterTrait;
    public $type = 'date';

    public $dateType = 'date_time';

    public $default = '';

    public $format = '';

    /**
     * getDateMakeMap
     *
     * @return array
     */
    public function getDateMakeMap()
    {
        return [
            'date'      => function ($type) {
                return $this->dateBind($type);
            },
            'year'      => function ($type) {
                return $this->yearBind($type);
            },
            'date_time' => function ($type) {
                return $this->dateTimeBind($type);
            }
        ];
    }

    /**
     * 绑定数据到当前对象
     *
     * @return mixed
     */
    public function bind()
    {
        $this->bindDateType();
        return $this;
    }

    public function bindDateType()
    {
        if (!empty($this->option['type'])) {
            $type           = key($this->option['type']);
            $this->dateType = $type;
            $this->bindOption();
        }
    }

    public function bindOption()
    {
        //
    }

    public function bindModel()
    {
        DB::transaction(function () {
            (new Model((new ModelProperties()), $this->modelKey, $this))
                ->writeDb(function () {
                    $this->make();
                })->checkTable();
        });
        return true;
    }

    /**
     * make
     */
    protected function make()
    {
        $type = $this->dateType;
        $map  = $this->getDateMakeMap();
        if (isset($map[$type]) && $map[$type] instanceof \Closure) {
            return $map[$type]($type);
        }
        return null;
    }

    /**
     * getDefault
     *
     * @return string
     * @throws PropertyException
     */
    public function getDefault()
    {
        $conf = Conf::conf()['option']['type'];
        if (isset($conf[$this->dateType]) && !empty($this->option['type'][$this->dateType])) {
            $this->default = !$this->checkDefault($this->option['type'][$this->dateType]) ? $conf[$this->dateType] :
                $this->option['type'][$this->dateType];
        }

        if (empty($this->default)) {
            throw new PropertyException('date default wrong');
        }
        return $this->default;
    }

    /**
     * checkDefault
     *
     * @param $value
     * @return bool
     * @throws PropertyException
     */
    protected function checkDefault($value)
    {
        $wait_data = [
            'date' => $value
        ];
        $check     = [
            'date'      => ['date' => 'date|date_format:Y-m-d'],
            'year'      => ['date' => 'date|date_format:Y'],
            'date_time' => ['date' => 'date|date_format:Y-m-d H:i:s']
        ];
        if (!empty($check[$this->dateType])) {
            $validator = Validator::make($wait_data, $check[$this->dateType]);
            if (empty($validator->fails())) {
                //$error = $validator->getMessageBag();
                return true;
            }
        }
        throw new PropertyException('date default value exception');
    }

    public function dateBind($type)
    {
        $this->dateType = $type;
        return $this->execSchema('date');
    }

    public function yearBind($type)
    {
        $this->dateType = $type;
        return $this->execSchema('year');
    }

    public function dateTimeBind($type)
    {
        $this->dateType = $type;
        return $this->execSchema('dateTime');
    }

    public function execSchema($type)
    {
        Schema::table($this->modelKey, function (Blueprint $table) use ($type) {
            $table = $table->{$type}($this->getKey())
                ->default($this->getDefault())
                ->comment($this->name);
            if ($this->alter) {
                /**
                 * @var ColumnDefinition $table
                 */
                $table->change();
            }
        });
        return $this;
    }

    /**
     * update
     *
     * @return bool
     */
    public function update()
    {
        DB::transaction(function () {
            $model = new ModelProperties();
            (new Model($this->getPropertyModelByKey($model), $this->modelKey, $this))
                ->updateDb(function () {
                    (new Alter())->getPropertyAlterByKey($this->type)
                        ->setProperty($this)->make()->make();
                })->checkTable();
        });
        return true;
    }
}