<?php

namespace Center\Capsule\Services\Property\Build;


use Center\Capsule\Models\ModelProperties;
use Center\Capsule\Models\Models;
use Center\Capsule\Services\Property\Exception\PropertyException;
use Illuminate\Support\Facades\Schema;

class Model
{
    /**
     * @var ModelProperties $model
     */
    public $model = '';

    public $modelKey = '';

    public $property;

    public $modelId = 0;

    public $foreignKey = 'model_id';

    /**
     * @var bool 系统初始化固定字段
     */
    private $systematic = false;

    public $allowMap = [
        'property_key'    => 'key',
        'property_type'   => 'type',
        'property_name'   => 'name',
        'property_option' => 'option',
        'property_filter' => 'filter',
        'widget'          => 'widget',
        'visibility'      => 'visibility'
    ];

    public function setSystematic($status)
    {
        $this->systematic = $status;
        return $this;
    }

    /**
     * Model constructor.
     *
     * @param $model
     * @param $modelKey
     * @param $property
     * @throws PropertyException
     */
    public function __construct($model, $modelKey, $property)
    {
        $this->model    = $model;
        $this->modelKey = $modelKey;
        $this->property = $property;
        $this->modelId  = $this->getModelId();
    }


    /**
     * writeDb
     *
     * @param \Closure $next
     * @return $this
     * @throws PropertyException
     */
    public function writeDb(\Closure $next)
    {
        $this->model->{$this->foreignKey} = $this->modelId;
        foreach ($this->allowMap as $k => $v) {
            if (property_exists($this->property, $v)) {
                $this->model->{$k} = $this->property->{$v};
            }
        }
        $this->checkKey();
        $next();
        $this->model->save();
        return $this;
    }

    /**
     * checkKey
     *
     * @return ModelProperties
     * @throws PropertyException
     */
    private function checkKey()
    {
        if (empty($this->model->getKey())) {
            //自定义key校验唯一
            $count = $this->model::query()->where('property_key', $this->property->key)
                ->where($this->foreignKey, $this->modelId)->count();
            if ($count > 0) {
                throw new PropertyException("The key value has been occupied");
            }
        }
        if (!$this->checkKeyRegex() || $this->checkEmbedField()) {
            throw new PropertyException("The key value is invalid or conflict embed key");
        }
        return $this->model;
    }

    /**
     * checkEmbedField
     *
     * @return bool
     */
    private function checkEmbedField()
    {
        if(!$this->systematic){
            if(isset(Models::$embedField[$this->property->key])){
                return true;
            }
        }
        return false;
    }

    /**
     * checkKeyRegex
     *
     * @return bool
     */
    private function checkKeyRegex()
    {
        if (mb_strlen($this->property->key) <= 50) {
            if (preg_match('/^[a-z]+[0-9]?+_?[a-z_0-9]+$/', $this->property->key)) {
                return true;
            }
        }
        return false;
    }

    /**
     * getModelId
     *
     * @return mixed
     * @throws PropertyException
     */
    public function getModelId()
    {
        $id = Models::where('model_key', $this->modelKey)->value('id');
        if (empty($id)) {
            throw new PropertyException('model Id exception');
        }
        return $id;
    }

    /**
     * checkTable
     *
     * @throws \Exception
     */
    public function checkTable()
    {
        $prefix  = config('database.connections.mysql.prefix');
        $columns = Schema::getConnection()->getDoctrineSchemaManager()
            ->listTableColumns($prefix . $this->modelKey);
        if (empty($columns[$this->property->key])) {
            throw new \Exception("field create fail");
        }
    }

    /**
     * updateDb
     *
     * @param \Closure $next
     * @return $this
     * @throws PropertyException
     */
    public function updateDb(\Closure $next)
    {
        //可以修改的属性
        $this->allowMap = array_diff($this->allowMap, $this->property->changeList);
        $this->writeDb($next);
        return $this;
    }
}