<?php

namespace Center\Capsule\Services\Property\Build;

use Center\Capsule\Constants\PropertyConst;
use Center\Capsule\Models\ModelProperties;
use Center\Capsule\Services\Property\Exception\PropertyException;
use Closure;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;
use Center\Capsule\Services\Property\Preset\Conf\NumberProperty as Conf;

class NumberProperty extends BaseProperty
{
    public $type = 'number';

    public $numberType = 'int';

    public $unsigned = false;

    public $point = 10;

    public $digit = 2;


    /**
     * getInterMap
     *
     * @param $type
     * @return string|string[]
     */
    public function getInterMap($type)
    {
        $map = [
            "int"      => 'bigInteger',
            "tinyint"  => 'tinyInteger',
            "smallint" => 'smallInteger'
        ];
        if (isset($type)) {
            return isset($map[$type]) ? $map[$type] : '';
        }
        return $map;
    }

    /**
     * getNumberMakeMap
     *
     * @return Closure[]
     */
    public function getNumberMakeMap()
    {
        return [
            'decimal'  => function () {
                return $this->decimalType();
            },
            'int'      => function () {
                return $this->intType();
            },
            'tinyint'  => function () {
                return $this->intType();
            },
            'smallint' => function () {
                return $this->intType();
            }
        ];
    }

    /**
     * getNumberTypeMap
     *
     * @return Closure[]
     */
    public function getNumberTypeMap()
    {
        return [
            'decimal' => function () {
                return $this->decimalBind();
            },
            'inter'   => function () {
                return $this->intBind();
            }
        ];
    }

    /**
     * 绑定数据到当前对象
     *
     * @return $this|mixed
     * @throws PropertyException
     */
    public function bind()
    {
        $this->bindNumberType();
        $this->bindSign();
        return $this;
    }

    /**
     * bindSign
     *
     * @return $this
     * @throws PropertyException
     */
    public function bindSign()
    {
        if (isset($this->option['unsigned'])) {
            $this->unsigned = $this->option['unsigned'] > 0;
        } else {
            throw new PropertyException("option.unsigned exception");
        }
        return $this;
    }

    /**
     * bindNumberType
     *
     * @throws PropertyException
     */
    public function bindNumberType()
    {
        if (!empty($this->option['type'])) {
            $type = key($this->option['type']);
            $map  = $this->getNumberTypeMap();
            if (isset($map[$type]) && $map[$type] instanceof Closure) {
                $map[$type]();
            } else {
                throw new PropertyException('No number type has been specified or defined');
            }
        }
    }

    /**
     * bindModel
     *
     * @return mixed|void
     */
    public function bindModel()
    {
        DB::transaction(function () {
            (new Model((new ModelProperties()), $this->modelKey, $this))
                ->writeDb(function () {
                    $this->make();
                })->checkTable();
        });
    }

    /**
     * make
     *
     * @throws PropertyException
     */
    public function make()
    {
        $type = $this->numberType;
        $map  = $this->getNumberMakeMap();
        if (isset($map[$type]) && $map[$type] instanceof Closure) {
            $map[$type]();
        } else {
            throw new PropertyException('No type has been specified or defined');
        }
    }

    /**
     * intType
     *
     */
    protected function intType()
    {
        Schema::table($this->modelKey, function (Blueprint $table) {
            $action = $this->getInterMap($this->numberType);
            $table->{$action}($this->getKey(), false, $this->unsigned)
                ->comment($this->name)
                ->default(0);
        });
    }

    /**
     * decimalType
     *
     */
    protected function decimalType()
    {
        Schema::table($this->modelKey, function (Blueprint $table) {
            $table->addColumn('decimal', $this->getKey(), [
                'total'    => $this->point,
                'places'   => $this->digit,
                'unsigned' => $this->unsigned
            ])->comment($this->name)
                ->default(0.00);
        });
    }

    /**
     * decimalBind
     *
     * @return $this
     * @throws PropertyException
     */
    public function decimalBind()
    {
        $this->numberType = 'decimal';
        foreach ($this->option['type'][$this->numberType] as $item => $v) {
            if (property_exists($this, $item)) {
                $this->{$item} = $v;
            }
        }
        $this->checkDecimal();
        return $this;
    }

    /**
     * checkDecimal
     *
     * @throws PropertyException
     */
    protected function checkDecimal()
    {
        $wait = [
            'point' => $this->point,
            'digit' => $this->digit
        ];
        foreach ($wait as $k => $v) {
            $int_options = [
                'options' => [
                    'min_range' => 1,
                    'max_range' => PropertyConst::$decimal[$k]['value']
                ]
            ];
            if (!filter_var($v, FILTER_VALIDATE_INT, $int_options)) {
                throw new PropertyException("decimal {$k} exception");
            }
            unset($int_options, $status);
        }

    }

    /**
     * intBind
     *
     * @return $this
     * @throws PropertyException
     */
    public function intBind()
    {
        if (isset($this->option['type']['inter']) &&
            count($this->option['type']['inter']) == 1) {
            $type                          = Conf::getOptionType();
            $this->numberType              = key($this->option['type']['inter']);
            $this->point                   = reset($this->option['type']['inter']);
            $this->option['type']['inter'] = [
                $this->numberType => $type['inter'][$this->numberType]
            ];
        } else {
            throw new PropertyException('number type option inter exception');
        }
        return $this;
    }

    /**
     * update
     *
     * @return bool|mixed
     */
    public function update()
    {
        DB::transaction(function () {
            $model = new ModelProperties();
            (new Model($this->getPropertyModelByKey($model), $this->modelKey, $this))
                ->updateDb(function () {
                    (new Alter())->getPropertyAlterByKey($this->type)
                        ->setProperty($this)->make();
                })->checkTable();
        });
        return true;
    }
}